import { signals, marketData, type Signal, type InsertSignal, type MarketData, type InsertMarketData, type User, type InsertUser } from "@shared/schema";

export interface IStorage {
  getUser(id: number): Promise<User | undefined>;
  getUserByUsername(username: string): Promise<User | undefined>;
  createUser(user: InsertUser): Promise<User>;
  
  // Signal operations
  createSignal(signal: InsertSignal): Promise<Signal>;
  getRecentSignals(limit?: number): Promise<Signal[]>;
  updateSignalResult(id: number, result: 'WIN' | 'LOSS', pnl: number): Promise<Signal | undefined>;
  
  // Market data operations
  updateMarketData(data: InsertMarketData): Promise<MarketData>;
  getMarketData(pair: string): Promise<MarketData | undefined>;
  getAllMarketData(): Promise<MarketData[]>;
}

export class MemStorage implements IStorage {
  private users: Map<number, User>;
  private signals: Map<number, Signal>;
  private marketData: Map<string, MarketData>;
  private currentUserId: number;
  private currentSignalId: number;
  private currentMarketDataId: number;

  constructor() {
    this.users = new Map();
    this.signals = new Map();
    this.marketData = new Map();
    this.currentUserId = 1;
    this.currentSignalId = 1;
    this.currentMarketDataId = 1;
    
    // Initialize with some market data
    this.initializeMarketData();
  }
  
  private initializeMarketData() {
    const pairs = ['EUR/USD', 'GBP/USD', 'USD/JPY', 'AUD/USD', 'USD/CAD', 'GBP/JPY', 'BTC/USD OTC'];
    const basePrices: { [key: string]: number } = {
      'EUR/USD': 1.08945,
      'GBP/USD': 1.24782,
      'USD/JPY': 148.234,
      'AUD/USD': 0.67123,
      'USD/CAD': 1.36789,
      'GBP/JPY': 184.567,
      'BTC/USD OTC': 43245,
    };
    
    pairs.forEach(pair => {
      const price = basePrices[pair] || 1.0;
      const change = (Math.random() - 0.5) * (price * 0.001);
      const changePercent = (change / price) * 100;
      
      this.marketData.set(pair, {
        id: this.currentMarketDataId++,
        pair,
        price,
        change,
        changePercent,
        timestamp: new Date(),
      });
    });
  }

  async getUser(id: number): Promise<User | undefined> {
    return this.users.get(id);
  }

  async getUserByUsername(username: string): Promise<User | undefined> {
    return Array.from(this.users.values()).find(
      (user) => user.username === username,
    );
  }

  async createUser(insertUser: InsertUser): Promise<User> {
    const id = this.currentUserId++;
    const user: User = { ...insertUser, id };
    this.users.set(id, user);
    return user;
  }

  async createSignal(insertSignal: InsertSignal): Promise<Signal> {
    const id = this.currentSignalId++;
    const signal: Signal = {
      ...insertSignal,
      id,
      timestamp: new Date(),
      result: null,
      pnl: null,
      martingaleStep: insertSignal.martingaleStep || 1,
    };
    this.signals.set(id, signal);
    return signal;
  }

  async getRecentSignals(limit: number = 50): Promise<Signal[]> {
    const allSignals = Array.from(this.signals.values());
    return allSignals
      .sort((a, b) => b.timestamp.getTime() - a.timestamp.getTime())
      .slice(0, limit);
  }

  async updateSignalResult(id: number, result: 'WIN' | 'LOSS', pnl: number): Promise<Signal | undefined> {
    const signal = this.signals.get(id);
    if (signal) {
      signal.result = result;
      signal.pnl = pnl;
      this.signals.set(id, signal);
    }
    return signal;
  }

  async updateMarketData(data: InsertMarketData): Promise<MarketData> {
    const existing = Array.from(this.marketData.values()).find(md => md.pair === data.pair);
    
    if (existing) {
      existing.price = data.price;
      existing.change = data.change;
      existing.changePercent = data.changePercent;
      existing.timestamp = new Date();
      this.marketData.set(data.pair, existing);
      return existing;
    } else {
      const id = this.currentMarketDataId++;
      const marketDataEntry: MarketData = {
        ...data,
        id,
        timestamp: new Date(),
      };
      this.marketData.set(data.pair, marketDataEntry);
      return marketDataEntry;
    }
  }

  async getMarketData(pair: string): Promise<MarketData | undefined> {
    return this.marketData.get(pair);
  }

  async getAllMarketData(): Promise<MarketData[]> {
    return Array.from(this.marketData.values());
  }
}

export const storage = new MemStorage();
